# Taken from fix included in later Buildbot, see
# https://github.com/buildbot/buildbot/commit/c325859efc1d33644e0c7954e41af25cb95c1078/Buildbot
# (http://goo.gl/OPynK).
from zope.interface import implements
import twisted.internet.interfaces

class NullAddress(object):
    "an address for NullTransport"
    implements(twisted.internet.interfaces.IAddress)

class NullTransport(object):
    "a do-nothing transport to make NetstringReceiver happy"
    implements(twisted.internet.interfaces.ITransport)
    def write(self, data): raise NotImplementedError
    def writeSequence(self, data): raise NotImplementedError
    def loseConnection(self): pass
    def getPeer(self):
        return NullAddress
    def getHost(self):
        return NullAddress

def hack_LogFileScanner__init__():
    """Fix http://trac.buildbot.net/ticket/1697"""
    from buildbot.status.builder import LogFileScanner
    original_init = LogFileScanner.__init__
    def __init__(self, *args, **kws):
        original_init(self, *args, **kws)
        self.makeConnection(NullTransport())
    LogFileScanner.__init__ = __init__

def hack_BuildStatus_getResults():
    """Make builder status that is completed not have a "results" of None.

    This fixes an XMLRPC problem.
    """
    import buildbot.status.builder
    def getResults(self):
        result = self.results
        if result is None and self.finished:
            result = buildbot.status.builder.FAILURE
        return result
    buildbot.status.builder.BuildStatus.getResults = getResults

def hack_XMLRPCServer__init__():
    """Prevent serialization error when retrieving historical information."""
    import buildbot.status.web.xmlrpc
    from twisted.web import xmlrpc
    def __init__(self):
        xmlrpc.XMLRPC.__init__(self)
        self.allowNone=True
    buildbot.status.web.xmlrpc.XMLRPCServer.__init__ = __init__

def install():
    hack_BuildStatus_getResults()
    hack_XMLRPCServer__init__()
    hack_LogFileScanner__init__()
