from bzrbuildbot import subunittest
import buildbot.process.buildstep
from buildbot.status.builder import SUCCESS, FAILURE
import buildbot.steps.shell


class TestObserver(
        subunittest.SubunitObserver,
        buildbot.process.buildstep.LogLineObserver):

    def __init__(self, *args, **kws):
        buildbot.process.buildstep.LogLineObserver.__init__(
            self, *args, **kws)
        subunittest.SubunitObserver.__init__(self)


class Test(buildbot.steps.shell.ShellCommand):

    progressMetrics = ('output', 'tests')

    def __init__(self, **kwargs):
        if 'description' not in kwargs:
            kwargs['description'] = ['testing']
        if 'descriptionDone' not in kwargs:
            kwargs['descriptionDone'] = ['test']
        buildbot.steps.shell.ShellCommand.__init__(self, **kwargs)
        self.observer = TestObserver()
        self.addLogObserver('stdio', self.observer)

    def describe(self, done=False):
        if done:
            self.observer.finish() # Close buildbot logs.
        description = buildbot.steps.shell.ShellCommand.describe(self, done)
        return subunittest.describe(self.step_status, description, done)

    def evaluateCommand(self, cmd):
        if self.observer.failure_count or self.observer.error_count:
            return FAILURE
        else:
            return SUCCESS
